import mxnet as mx
from mxnet import gluon

from .constants import DATA_TYPE
from .gluon_blocks_helpers import LogarithmScalarEncoding, PositiveScalarEncoding, encode_unwrap_parameter


def param_to_pretty_string(gluon_param, encoding):
    """
    Take a gluon parameter and transform it to a string amenable to plotting
    If need be, the gluon parameter is appropriately encoded (e.g., log-exp transform).

    :param gluon_param: gluon parameter
    :param encoding: object in charge of encoding/decoding the gluon_param
    """

    assert isinstance(gluon_param, gluon.Parameter)
    assert encoding is not None, \
        "encoding of param {} should not be None".format(gluon_param.name)

    param_as_numpy = encode_unwrap_parameter(
        mx.nd, gluon_param, encoding).asnumpy()
    return "{}: {}".format(
        gluon_param.name, ";".join(
            "{:.6f}".format(value) for value in param_as_numpy))


def create_encoding(
        encoding_name, init_val, constr_lower, constr_upper, dimension, prior):
    assert encoding_name in ['logarithm', 'positive'], \
        "encoding name can only be 'logarithm' or 'positive'"

    if encoding_name == 'logarithm':
        return LogarithmScalarEncoding(init_val=init_val,
                                       constr_lower=constr_lower,
                                       constr_upper=constr_upper,
                                       dimension=dimension,
                                       regularizer=prior)
    else:
        return PositiveScalarEncoding(lower=constr_lower,
                                      init_val = init_val,
                                      constr_upper=constr_upper,
                                      dimension=dimension,
                                      regularizer=prior)


PARAMETER_POSTFIX = '_internal'


def get_name_internal(name):
    return name + PARAMETER_POSTFIX


def register_parameter(
        params, name, encoding, shape=(1,), dtype=DATA_TYPE):
    return params.get(
        get_name_internal(name), shape=shape,
        init=mx.init.Constant(encoding.init_val_int), dtype=dtype)



from .profile_api import MLSProfileAPIClient
from .recommendation_api import MLSRecommendationAPIClient
from .conversion_tracking_api import MLSConversionTrackingAPIClient

__all__ = ["MLSProfileAPIClient", "MLSRecommendationAPIClient", "MLSConversionTrackingAPIClient"]



from sktmls.models.contrib import MbrPopularBenefitModel


class TestMbrPopularBenefitModel:
    def test_000_mbr_valid_1(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "benefit_ranking_weekday_hour_00": ["0", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_01": ["1", "0", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_02": ["2", "1", "0", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_03": ["3", "1", "2", "0", "4", "5", "6"],
                "benefit_ranking_weekday_hour_04": ["4", "1", "2", "3", "0", "5", "6"],
                "benefit_ranking_weekday_hour_05": ["5", "1", "2", "3", "4", "0", "6"],
                "benefit_ranking_weekday_hour_06": ["6", "1", "2", "3", "4", "5", "0"],
                "benefit_ranking_weekday_hour_07": ["7", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_08": ["8", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_09": ["9", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_10": ["10", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_11": ["11", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_12": ["12", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_13": ["13", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_14": ["14", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_15": ["15", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_16": ["16", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_17": ["17", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_18": ["18", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_19": ["19", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_20": ["20", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_21": ["21", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_22": ["22", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_23": ["23", "1", "2", "3", "4", "5", "6"],
            },
        )

        mbm = MbrPopularBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "sex_cd",
                "age",
                "mbr_card_gr_cd",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            hourly_contexts=[i for i in range(24)],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            item_type="test_type",
            num_pick=1,
        )

        sex_cd = "1"
        age = 32
        mbr_card_gr_cd = "V"
        available_benefit_activity = ["103", "c21", "a3", "b4"]
        available_benefit_asianfood = None
        available_benefit_beauty = ["1", "104", "320"]

        results = mbm.predict(
            [
                "user_id",
                sex_cd,
                age,
                mbr_card_gr_cd,
                available_benefit_activity,
                available_benefit_asianfood,
                available_benefit_beauty,
            ]
        )

        assert len(results["items"]) == 1

        item = results["items"][0]
        assert item["id"] == "1"
        assert item["type"] == "test_type"

    def test_001_mbr_valid_2(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "benefit_ranking_weekday_hour_00": ["0", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_01": ["1", "0", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_02": ["2", "1", "0", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_03": ["3", "1", "2", "0", "4", "5", "6"],
                "benefit_ranking_weekday_hour_04": ["4", "1", "2", "3", "0", "5", "6"],
                "benefit_ranking_weekday_hour_05": ["5", "1", "2", "3", "4", "0", "6"],
                "benefit_ranking_weekday_hour_06": ["6", "1", "2", "3", "4", "5", "0"],
                "benefit_ranking_weekday_hour_07": ["7", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_08": ["8", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_09": ["9", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_10": ["10", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_11": ["11", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_12": ["12", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_13": ["13", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_14": ["14", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_15": ["15", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_16": ["16", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_17": ["17", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_18": ["18", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_19": ["19", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_20": ["20", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_21": ["21", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_22": ["22", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_23": ["23", "1", "2", "3", "4", "5", "6"],
            },
        )

        mbm = MbrPopularBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "sex_cd",
                "age",
                "mbr_card_gr_cd",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            hourly_contexts=[i for i in range(24)],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            item_type="test_type",
            num_pick=10,
        )

        sex_cd = "1"
        age = 32
        mbr_card_gr_cd = "V"
        available_benefit_activity = ["103", "210", "a3", "4"]
        available_benefit_asianfood = None
        available_benefit_beauty = ["1", "104", "200"]

        results = mbm.predict(
            [
                "user_id",
                sex_cd,
                age,
                mbr_card_gr_cd,
                available_benefit_activity,
                available_benefit_asianfood,
                available_benefit_beauty,
            ]
        )

        assert len(results["items"]) == 2

        item = results["items"][0]
        assert item["type"] == "test_type"

        item = results["items"][1]
        assert item["type"] == "test_type"

    def test_002_mbr_invalid_condition(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "benefit_ranking_weekday_hour_00": ["0", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_01": ["1", "0", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_02": ["2", "1", "0", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_03": ["3", "1", "2", "0", "4", "5", "6"],
                "benefit_ranking_weekday_hour_04": ["4", "1", "2", "3", "0", "5", "6"],
                "benefit_ranking_weekday_hour_05": ["5", "1", "2", "3", "4", "0", "6"],
                "benefit_ranking_weekday_hour_06": ["6", "1", "2", "3", "4", "5", "0"],
                "benefit_ranking_weekday_hour_07": ["7", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_08": ["8", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_09": ["9", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_10": ["10", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_11": ["11", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_12": ["12", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_13": ["13", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_14": ["14", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_15": ["15", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_16": ["16", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_17": ["17", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_18": ["18", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_19": ["19", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_20": ["20", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_21": ["21", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_22": ["22", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_23": ["23", "1", "2", "3", "4", "5", "6"],
            },
        )

        sex_cd = "3"
        age = 32
        mbr_card_gr_cd = "V"

        available_benefit_activity = ["103", "c21", "3", "4"]
        available_benefit_asianfood = None
        available_benefit_beauty = ["a1", "b104", "c20"]

        mbm = MbrPopularBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "sex_cd",
                "age",
                "mbr_card_gr_cd",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            hourly_contexts=[i for i in range(24)],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            item_type="test_type",
            num_pick=10,
        )

        results = mbm.predict(
            [
                "user_id",
                sex_cd,
                age,
                mbr_card_gr_cd,
                available_benefit_activity,
                available_benefit_asianfood,
                available_benefit_beauty,
            ]
        )

        assert len(results["items"]) == 0

    def test_003_mbr_valid_all_none(self, mocker):
        mocker.patch(
            "sktmls.apis.MLSProfileAPIClient.get_item_profile",
            return_value={
                "benefit_ranking_weekday_hour_00": ["0", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_01": ["1", "0", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_02": ["2", "1", "0", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_03": ["3", "1", "2", "0", "4", "5", "6"],
                "benefit_ranking_weekday_hour_04": ["4", "1", "2", "3", "0", "5", "6"],
                "benefit_ranking_weekday_hour_05": ["5", "1", "2", "3", "4", "0", "6"],
                "benefit_ranking_weekday_hour_06": ["6", "1", "2", "3", "4", "5", "0"],
                "benefit_ranking_weekday_hour_07": ["7", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_08": ["8", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_09": ["9", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_10": ["10", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_11": ["11", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_12": ["12", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_13": ["13", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_14": ["14", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_15": ["15", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_16": ["16", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_17": ["17", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_18": ["18", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_19": ["19", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_20": ["20", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_21": ["21", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_22": ["22", "1", "2", "3", "4", "5", "6"],
                "benefit_ranking_weekday_hour_23": ["23", "1", "2", "3", "4", "5", "6"],
            },
        )

        sex_cd = "1"
        age = 32
        mbr_card_gr_cd = "V"
        available_benefit_activity = ["103", "c21", "a3", "b4"]
        available_benefit_asianfood = None
        available_benefit_beauty = ["a1", "b104", "c20"]

        mbm = MbrPopularBenefitModel(
            model_name="test_model",
            model_version="test_version",
            features=[
                "user_id",
                "sex_cd",
                "age",
                "mbr_card_gr_cd",
                "available_benefit_activity",
                "available_benefit_asianfood",
                "available_benefit_beauty",
            ],
            hourly_contexts=[i for i in range(24)],
            categories=["mbr_activity", "mbr_asianfood", "mbr_beauty"],
            item_type="test_type",
            num_pick=1,
        )

        results = mbm.predict(
            [
                "user_id",
                sex_cd,
                age,
                mbr_card_gr_cd,
                available_benefit_activity,
                available_benefit_asianfood,
                available_benefit_beauty,
            ]
        )

        assert len(results["items"]) == 0



from typing import List, TYPE_CHECKING

from dateutil import parser

from sktmls import MLSClient, MLSENV, MLSRuntimeENV, MLSResponse, MLSClientError

if TYPE_CHECKING:
    from sktmls.experiments import Experiment

MLS_CHANNELS_API_URL = "/api/v1/channels"


class Channel:
    """
    MLS 채널 클래스입니다.
    """

    def __init__(self, **kwargs):
        """
        ## Args

        - kwargs
            - id: (int) 채널 고유 ID
            - screen_id: (str) 채널 이름
            - experiments: (list(int)) 연결된 실험 ID 리스트
            - user: (str) 채널 생성 계정명
            - created_at: (datetime) 생성일시
            - updated_at: (datetime) 수정일시

        ## Returns
        `sktmls.channels.Channel`
        """
        self.id = kwargs.get("id")
        self.screen_id = kwargs.get("screen_id")
        self.experiments = kwargs.get("experiments")
        self.user = kwargs.get("user")

        if kwargs.get("created_at"):
            self.created_at = parser.parse(kwargs.get("created_at"))
        if kwargs.get("updated_at"):
            self.updated_at = parser.parse(kwargs.get("updated_at"))

    def get(self):
        return self.__dict__

    def reset(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class ChannelClient(MLSClient):
    """
    MLS 채널 관련 기능들을 제공하는 클라이언트입니다.
    """

    def __init__(
        self,
        env: MLSENV = None,
        runtime_env: MLSRuntimeENV = None,
        username: str = None,
        password: str = None,
    ):
        """
        ## Args

        - env: (`sktmls.MLSENV`) 접근할 MLS 환경 (`sktmls.MLSENV.DEV`|`sktmls.MLSENV.STG`|`sktmls.MLSENV.PRD`) (기본값: `sktmls.MLSENV.STG`)
        - runtime_env: (`sktmls.MLSRuntimeENV`) 클라이언트가 실행되는 환경 (`sktmls.MLSRuntimeENV.YE`|`sktmls.MLSRuntimeENV.EDD`|`sktmls.MLSRuntimeENV.LOCAL`) (기본값: `sktmls.MLSRuntimeENV.LOCAL`)
        - username: (str) MLS 계정명 (기본값: $MLS_USERNAME)
        - password: (str) MLS 계정 비밀번호 (기본값: $MLS_PASSWORD)

        아래의 환경 변수가 정의된 경우 해당 파라미터를 생략 가능합니다.

        - $MLS_ENV: env
        - $MLS_RUNTIME_ENV: runtime_env
        - $MLS_USERNAME: username
        - $MLS_PASSWORD: password

        ## Returns
        `sktmls.channels.ChannelClient`

        ## Example

        ```python
        channel_client = ChannelClient(env=MLSENV.STG, runtime_env=MLSRuntimeENV.YE, username="mls_account", password="mls_password")
        ```
        """

        super().__init__(env=env, runtime_env=runtime_env, username=username, password=password)

    def create_channel(self, name: str, experiments: List["Experiment"]) -> Channel:
        """
        채널를 생성합니다.

        ## Args

        - name: (str) 채널 이름
        - experiments: (list(`sktmls.experiments.Experiment`)) 실험 객체 리스트

        ## Returns
        `sktmls.channels.Channel`

        - id: (int) 채널 고유 ID
        - screen_id: (str) 채널 이름
        - experiments: (list(int)) 연결된 실험 ID 리스트
        - user: (str) 채널 생성 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        experiment_1 = experiment_client.get_experiment(name="my_experiment_1")
        experiment_2 = experiment_client.get_experiment(name="my_experiment_2")

        channel = channel_client.create_channel(
            name="my_channel",
            experiments=[
                experiment_1,
                experiment_2,
            ]
        )
        ```
        """
        assert type(experiments) == list, "`experiments`는 Experiemnt객체를 담은 list타입이어야 합니다."
        assert len(experiments) >= 1, "최소 1개 이상의 Experiment객체가 포함되어야 합니다."

        data = {
            "screen_id": name,
            "experiments": [experiment.id for experiment in experiments],
        }

        return Channel(**self._request(method="POST", url=MLS_CHANNELS_API_URL, data=data).results)

    def list_channels(self, **kwargs) -> List[Channel]:
        """
        채널 리스트를 가져옵니다. (채널 조건 제외)

        ## Args

        - kwargs: (optional) (dict) 채널 조건
            - id: (int) 채널 고유 ID
            - name: (str) 채널 이름
            - query: (str) 검색 문자
            - page: (int) 페이지 번호

        ## Returns
        list(`sktmls.channels.Channel`)

        - id: (int) 채널 고유 ID
        - name: (str) 채널 이름
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        channels = channel_client.list_channels()
        ```
        """

        response = self._request(method="GET", url=f"{MLS_CHANNELS_API_URL}", params=kwargs).results

        return [Channel(**channel) for channel in response]

    def get_channel(self, id: int = None, name: str = None) -> Channel:
        """
        채널 정보를 가져옵니다.

        ## Args: `id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다.

        - id: (optional) (int) 채널 고유 ID
        - name: (optional) (str) 채널 이름

        ## Returns
        `sktmls.channels.Channel`

        - id: (int) 채널 고유 ID
        - screen_id: (str) 채널 이름
        - experiments: (list(int)) 연결된 실험 ID 리스트
        - user: (str) 채널 생성 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        channel_by_id = channel_client.get_channel(
            id=3
        )
        channel_by_name = channel_client.get_channel(
            name="my_channel"
        )
        ```
        """
        assert id or name, "`id` 또는 `name` 중 한 개 이상의 값이 반드시 전달되어야 합니다."

        channels = self.list_channels(id=id, screen_id=name)
        if len(channels) == 0:
            raise MLSClientError(code=404, msg="채널이 없습니다.")
        elif len(channels) > 1:
            raise MLSClientError(code=409, msg="채널이 여러개 존재합니다.")
        return Channel(**self._request(method="GET", url=f"{MLS_CHANNELS_API_URL}/{channels[0].id}").results)

    def update_channel(self, channel: Channel, name: str = None, experiments: List["Experiment"] = None) -> Channel:
        """
        채널 정보를 수정합니다.

        ## Args

        - channel: (`sktmls.channels.Channel`) 채널 객체
        - name: (optional) (str) 채널 이름
        - experiments: (optional) (list(`sktmls.experiments.Experiment`)) 실험 객체 리스트

        ## Returns
        `sktmls.channels.Channel`

        - id: (int) 채널 고유 ID
        - screen_id: (str) 채널 이름
        - experiments: (list(int)) 연결된 실험 ID 리스트
        - user: (str) 채널 생성 계정명
        - created_at: (datetime) 생성일시
        - updated_at: (datetime) 수정일시

        ## Example

        ```python
        experiment_new = experiment_client.get_experiment(name="my_experiment_new")

        channel = channel_clinet.get_channel(name="my_channel")
        channel = channel_client.update_channel(
            channel=channel,
            name="my_updated_channel",
            experiments=[experiment_new],
        )
        ```
        """
        assert type(channel) == Channel
        assert type(experiments) == list, "`experiments`는 Experiemnt객체를 담은 list타입이어야 합니다."
        assert len(experiments) >= 1, "최소 1개 이상의 Experiment객체가 포함되어야 합니다."

        data = {"screen_id": channel.screen_id, "experiments": channel.experiments}
        if name:
            data["screen_id"] = name
        if experiments:
            data["experiments"] = [experiment.id for experiment in experiments]

        channel.reset(**self._request(method="PUT", url=f"{MLS_CHANNELS_API_URL}/{channel.id}", data=data).results)
        return channel

    def delete_channel(self, channel: Channel) -> MLSResponse:
        """
        채널 삭제합니다.

        ## Args

        - channel: (`sktmls.channels.Channel`) 채널 객체

        ## Returns
        `sktmls.MLSResponse`

        ## Example

        ```python
        channel_client.delete_channel(channel)
        ```
        """

        assert type(channel) == Channel

        return self._request(method="DELETE", url=f"{MLS_CHANNELS_API_URL}/{channel.id}")
